import { jsx as _jsx, jsxs as _jsxs } from "@emotion/react/jsx-runtime";
/** @jsxImportSource @emotion/react */
import * as React from "react";
import PropTypes from "prop-types";
import cx from "classnames";
import { useContext, } from "react";
import { useMeasure } from "./Hooks/use-measure";
import { PanelEnlargeCtx, useMMenuBarOcup } from "./MainMenuBar";
//缺省breaks ： <LineColumn column={2} breaks={[288, 520]}
//可以安排几个列 :要每列相等宽度，每一行高度也一相等的，最大支持8列（8块一排的）有意义的用途：编辑器某项目？。不是表格某一列的用途
const defaultBreakOf = [
    [360],
    [330, 1000],
    [310, 800, 1300],
    [280, 580, 1000, 1400],
    [250, 500, 800, 1150, 1500],
    [220, 420, 680, 980, 1300, 1600],
    [200, 380, 670, 920, 1150, 1450, 1750],
    [170, 340, 590, 820, 1100, 1300, 1580, 1880], //默认的配置最多支持八个，但也能允许自己扩展更多个的。
];
/**【3种思路】如何避免依据屏幕宽度做动态布局的效果，在遇到打印切换时刻，打印预览太草率地限定了纸张页数所导致的问题。
 * 1：每个LineColumn出现地方都应该配置打印A4。A4横板的。A3?几个常见打印目的地纸张的正常应该的效果：也就是suitableCls大小{a4:,a4L:,a3,}。
 * 2：直接约束准备打印浏览器的窗口宽度做法。
 * 3：完整的用css来调整的？【确保打印的正确性需要】只用css，不用JS做动态内容？测量div宽度做自适应的危险；
 * 重新加载？打印敲定页数之后才会再一次render的。根本没机会计算ref.div最新宽度甚至用beforePrintHandler来setSuitableCls也没有机会提前调整。
 * printComp[]；
 * 手机上chrome打印可能有些问题；
 * 若使用固定大小列style={{columnWidth: '270px', columnGap: '20px'}}布局，大小屏幕宽度缩放适应性不好，水平方向没法对齐，感觉凌乱！。
 * 若采用style={{display: 'grid', gridTemplateColumns: 'auto auto auto'}}，也有问题：列数量不能固定啊。
 * 1：2 1：1 框架调整，右边还是左边的半边框，默认编辑区底下空间100%，屏幕媒体宽度。应该部署有几个列的？？
 * */
export const LineColumn = ({ column: defaultColumn = 4, children, breaks = defaultBreakOf[defaultColumn - 1], inLeft, percent, ...other }) => {
    const contextPanel = useContext(PanelEnlargeCtx);
    const { carousel } = useMMenuBarOcup(false);
    //（全部列的）Viewpoint屏占比 系数
    let vppercent;
    if (percent !== undefined)
        vppercent = percent; //不启用useContext(PanelEnlargeCtx)+useMMenuBarOcup+inLeft的设置，采信外部直接控制的。
    else {
        const { enlarge } = contextPanel || {};
        //enlarge两半边的宽度协调:左边没有=0，1：左边右边1比2， 2： 1比1， 3： 2比1， 4：右边没有。
        vppercent = inLeft ? (enlarge === 1 ? 0.33 : enlarge === 3 ? 0.66 : enlarge === undefined ? 1 : 0.5) :
            (enlarge === 1 ? 0.66 : enlarge === 3 ? 0.33 : enlarge === undefined ? 1 : 0.5);
        if (contextPanel === null || carousel)
            vppercent = 1; //没有拆分为两个半边页面的框架形式，独立全撑开的页面。
        // console.log("打enlarge触发vppercent=",vppercent,enlarge,carousel);
    }
    //最大屏幕 可放下 安排 最多几个列数的； 非线性地比例降低列数?
    const column = breaks.length;
    if (column < 1)
        throw new Error("number of columns must >=1");
    /**反过来预设赋值 倒推合理的断开点;
     * 依据vppercent断定合适的CSS,
     * */
    const mediaQueries = React.useCallback((vppercent) => {
        if (vppercent <= 0)
            return {
                visibility: 'hidden'
            };
        const breakv = breaks.map(px => Math.round(px / vppercent));
        let csobj = {};
        for (let p = 0; p < column - 1; p++) {
            let key = '@media (min-width: ' + breakv[p] + 'px) and (max-width: ' + (breakv[p + 1] - 1) + 'px)';
            let col = p === 0 ? 1 : p + 1;
            csobj[key] = {
                gridTemplateColumns: 'repeat(' + col + ', auto)',
            };
        }
        let key = '@media (min-width: ' + breakv[column - 1] + 'px)';
        csobj[key] = {
            gridTemplateColumns: 'repeat(' + column + ', auto)',
        };
        // console.log("mediaQueries会了vppercent=",vppercent,breaks, csobj);
        return csobj;
    }, [column, breaks]);
    return (_jsx("div", { css: {
            display: 'grid',
            gridTemplateColumns: 'auto',
            //【自适应】grid-template-columns: repeat(auto-fit, minmax(min(200px, 100%), 1fr));  列宽度固定的。宽屏幕窄屏幕没有区分对待。
            ...mediaQueries(vppercent),
        }, ...other, children: children }));
};
/**被动态网页的打印给折腾的：
 重新加载？打印敲定页数之后才会再一次render的。根本没机会计算ref.div最新宽度甚至用beforePrintHandler来setSuitableCls也没有机会提前调整。
 beforePrintHandler --》unmount this?重新加载
 react-to-print只会复制旧的组件页面也不能重新render的是甚至usemedia都无感知到打印。；
 【确保打印的正确性需要】只用css，不用JS做动态内容？测量div宽度做自适应的就会危险了；
 function validChildrenCount(children: any) {
 return React.Children.toArray(children).filter(child =>
 React.isValidElement(child)
 ).length;
 }
 * */
//在一行内部的空间分配比例 %：
// interface LineColumn_OLDProps extends LineColumnProps {
//     //编译器没办法支持: css={{flexWrap: 'nowrap !important'}},所以加一个参数了。
//     nowrap?: boolean;
// }
/**旧版本 用flexBox, 测量自己父辈容器div宽度来自适应的。
* */
export const LineColumnFlex = ({ column: defaultColumn = 4, children, breaks = defaultBreakOf[defaultColumn - 1], ...other }) => {
    const reflc = React.useRef(null);
    // const width=reflc?.current?.getBoundingClientRect()?.width || 0;        //假如useMeasure(reflc)?.width!;打印有问题；宽度感知落后。
    const width = useMeasure(reflc.current ? reflc : null)?.width || 0;
    //最适合搞几个列。
    const [suitableCls, setSuitableCls] = React.useState(2);
    //最大屏幕 可放下 安排 最多几个列数的； 非线性地比例降低列数?
    const column = breaks.length;
    if (column < 1)
        throw new Error("number of columns must >=1");
    // const printing = useMedia('print');     //React.cloneElement(传递的printing都是:====false的。打印不会看到true;
    //【为了打印正常不能改掉的】@ @注意！ useLayoutEffect； 假如改成useEffect无法正常打印预览列{suitableCls}宽{width}的连两个数字前面哪个不对了!。useLayoutEffect更新彻底，useEffect会需要两次刷新啊。
    React.useLayoutEffect(() => {
        let i = 0;
        for (; i < column; i++)
            if (width < breaks[i])
                break;
        setSuitableCls(i < 1 ? 1 : i);
    }, [width, column, setSuitableCls, breaks]);
    const itemCount = React.Children.count(children);
    const lineSum = Math.ceil(itemCount / suitableCls);
    //【打印的情形：printing没办法传递最新取自给儿子，还是旧的printing取值】  React.useMemo( ()也不行啊。
    const items = React.Children.map(children, (item, index) => {
        return React.cloneElement(item, {
            className: cx(['LineColumn-item',
                index === itemCount - 1 ? 'Litem-last' : ''
            ]),
            style: { flex: `1 1 ${Math.round(10000 / suitableCls) / 100}%` },
        });
    });
    const fakeSons = lineSum * suitableCls - itemCount;
    //必须加一层div{{ position: 'relative' }}，后面给出的父辈Div宽度width才对的。
    return (_jsxs("div", { ref: reflc, style: { display: 'flex', flexWrap: 'wrap', position: 'relative', justifyContent: 'center', width: '-webkit-fill-available' }, ...other, children: [items, fakeSons > 0 && (new Array(fakeSons)).fill(null).map((each, clm) => {
                return _jsx("div", { css: { flex: `1 1 ${Math.round(10000 / suitableCls) / 100}%` } }, clm);
            })] }));
};
LineColumn.displayName = "LineColumn";
LineColumn.propTypes = {
    children: PropTypes.node
};
//# sourceMappingURL=LineColumn.js.map